@@ -3,7 +3,7 @@ package Web::Microformats2;
 use Web::Microformats2::Parser;
 use Web::Microformats2::Document;
 
-our $VERSION = '0.4';
+our $VERSION = '0.5';
 
 1;
 
@@ -45,7 +45,7 @@ it as a queryable in-memory object. They can also serialize this object
 as JSON (using the Microformats2 rules for this), or read an already
 JSON-serialized Microformats2 structure for further analysis.
 
-See L<"ABOUT MICROFORMATS">, below, for arguments about why this might
+See L<"ABOUT MICROFORMATS2">, below, for arguments about why this might
 be interesting to you.
 
 =head1 CLASSES
@@ -75,17 +75,15 @@ of properties, and possibly a parent item and a list of child items.
 
 =head1 STATUS
 
-These modules are B<alpha>. Their developer is still figuring out how
-they want to be used, and their interface may change dramatically. This
-software is currently being "dogfooded" on an experimental branch of the
-author's own essay blog.
+These modules provide a I<reasonably complete> implementation of L<the
+Microformats2 Living
+Specification|http://microformats.org/wiki/microformats2-parsing>. They
+pass all of L<the official MF2 baseline test
+cases|https://github.com/microformats/tests>, a copy of which is
+included with these modules' own test suite.
 
-Further, the parser implementation is incomplete. The HTML and JSON
-documents found in t/microformats-v2 include the full Microformats2
-parser test suite found at L<https://github.com/microformats/tests>, as
-of early April, 2018. The parser-test program in t/parser.pl marks a
-great deal of these tests as TODO. (But fewer than half of them, for all
-that, so hey.)
+The author considers this software B<beta>. Its public interface may
+still change, but not without some effort at supporting its current API.
 
 =head1 ABOUT MICROFORMATS2
 
@@ -140,3 +138,45 @@ This software is Copyright (c) 2018 by Jason McIntosh.
 This is free software, licensed under:
 
   The MIT (X11) License
+
+=head1 A PERSONAL REQUEST
+
+My ability to share and maintain free, open-source software like this
+depends upon my living in a society that allows me the free time and
+personal liberty to create work benefiting people other than just myself
+or my immediate family. I recognize that I got a head start on this due
+to an accident of birth, and I strive to convert some of my unclaimed
+time and attention into work that, I hope, gives back to society in some
+small way.
+
+Worryingly, I find myself today living in a country experiencing a
+profound and unwelcome political upheaval, with its already flawed
+democracy under grave threat from powerful authoritarian elements. These
+powers wish to undermine this society, remolding it according to their
+deeply cynical and strictly zero-sum philosophies, where nobody can gain
+without someone else losing.
+
+Free and open-source software has no place in such a world. As such,
+these autocrats' further ascension would have a deleterious effect on my
+ability to continue working for the public good.
+
+Therefore, if you would like to financially support my work, I would ask
+you to consider a donation to one of the following causes. It would mean
+a lot to me if you did. (You can tell me about it if you'd like to, but
+you don't have to.)
+
+=over
+
+=item *
+
+L<The American Civil Liberties Union|https://aclu.org>
+
+=item *
+
+L<The Democratic National Committee|https://democrats.org>
+
+=item *
+
+L<Earthjustice|https://earthjustice.org>
+
+=back
