@@ -8,6 +8,7 @@ use v5.10;
 use Scalar::Util;
 use JSON;
 use DateTime::Format::ISO8601;
+use Carp;
 
 use Web::Microformats2::Item;
 use Web::Microformats2::Document;
@@ -56,12 +57,15 @@ sub parse {
 
 # analyze_element: Recursive method that scans an element for new microformat
 # definitions (h-*) or properties (u|dt|e|p-*) and then does the right thing.
+# It also builds up the MF2 document's rels and rel-urls as it goes.
 sub analyze_element {
     my $self = shift;
     my ( $document, $element, $current_item ) = @_;
 
     return unless blessed( $element) && $element->isa( 'HTML::Element' );
 
+    $self->_add_element_rels_to_mf2_document( $element, $document );
+
     my $mf2_attrs = $self->_tease_out_mf2_attrs( $element );
 
     my $h_attrs = delete $mf2_attrs->{h};
@@ -89,19 +93,12 @@ sub analyze_element {
             # this property name.)
             unless ( $new_item ) {
                 for my $property ( @$properties_ref ) {
-                    my $vcp_fragments_ref =
-                        $self->_seek_value_class_pattern( $element );
-                    if ( @$vcp_fragments_ref ) {
+                    my $value = $self->_parse_property_value( $element );
+                    if ( defined $value ) {
                         $current_item->add_property(
                             "p-$property",
-                            join q{}, @$vcp_fragments_ref,
-                        )
-                    }
-                    elsif ( my $alt = $element->findvalue( './@title|@value|@alt' ) ) {
-                        $current_item->add_property( "p-$property", $alt );
-                    }
-                    elsif ( my $text = _trim( decode_entities($element->as_text) ) ) {
-                        $current_item->add_property( "p-$property", $text );
+                            $value,
+                        );
                     }
                 }
             }
@@ -187,7 +184,7 @@ sub analyze_element {
                 my $vcp_fragments_ref =
                     $self->_seek_value_class_pattern( $element );
                 if ( @$vcp_fragments_ref ) {
-                    $dt_string = join q{}, @$vcp_fragments_ref;
+                    $dt_string = $self->_format_datetime(join (q{T}, @$vcp_fragments_ref), $current_item);
                 }
                 elsif ( my $alt = $element->findvalue( './@datetime|@title|@value' ) ) {
                     $dt_string = $alt;
@@ -196,17 +193,9 @@ sub analyze_element {
                     $dt_string = $text;
                 }
                 if ( defined $dt_string ) {
-                    my $dt;
-                    eval {
-                    $dt = DateTime::Format::ISO8601->new
-                              ->parse_datetime( $dt_string );
-                    };
-                    next if $@;
-                    # XXX Needs to check for & set timezone offset
-                    my $format = '%Y-%m-%d %H:%M:%S';
                     $current_item->add_property(
                         "dt-$property",
-                        $dt->strftime( $format ),
+                        $dt_string,
                     );
                 }
             }
@@ -230,13 +219,21 @@ sub analyze_element {
 
         # Now add a "value" attribute to this new item, if appropriate,
         # according to the MF2 spec.
+        my $value_attribute;
         if ( $mf2_attrs->{p}->[0] ) {
-            $new_item->value( $new_item->get_properties('name')->[0] );
+            if ( my $name = $new_item->get_properties('name')->[0] ) {
+                $value_attribute = $name;
+            }
+            else {
+                $value_attribute = $self->_parse_property_value( $element );
+            }
         }
         elsif ( $mf2_attrs->{u}->[0] ) {
-            $new_item->value( $new_item->get_properties('url')->[0] );
+            $value_attribute = $new_item->get_properties('url')->[0];
         }
 
+        $new_item->value( $value_attribute ) if defined ($value_attribute);
+
         # Put this onto the parent item's property-list, or its children-list,
         # depending on context.
         my $item_property;
@@ -341,7 +338,7 @@ sub _set_implied_name {
 
     my $types = $item->types;
 
-    return if $item->has_p_properties || $item->has_e_properties;
+    return if $item->has_properties || $item->has_children;
 
     my $xpath;
     my $name;
@@ -520,6 +517,9 @@ sub _seek_value_class_pattern {
         if ( $1 ) {
             push @$vcp_fragments_ref, $element->attr( 'title' );
         }
+        elsif ( ( $element->tag =~ /^(del|ins|time)$/ ) && defined( $element->attr('datetime'))) {
+            push @$vcp_fragments_ref, $element->attr('datetime');
+        }
         else {
             my $html;
             for my $content_piece ( $element->content_list ) {
@@ -551,6 +551,153 @@ sub _trim {
     return $string;
 }
 
+sub _format_datetime {
+    my ($self, $dt_string, $current_item) = @_;
+
+    my $dt;
+
+    # Knock off leading/trailing whitespace.
+    $dt_string = _trim($dt_string);
+
+    $dt_string =~ s/t/T/;
+
+    # Note presence of AM/PM, but toss it out of the string.
+    $dt_string =~ s/((?:a|p)\.?m\.?)//i;
+    my $am_or_pm = $1 || '';
+
+    # Store the provided TZ offset.
+    my ($provided_offset) = $dt_string =~ /([\-\+Z](?:\d\d:?\d\d)?)$/;
+    $provided_offset ||= '';
+
+    # Reformat HHMM offset as HH:MM.
+    $dt_string =~ s/(-|\+)(\d\d)(\d\d)/$1$2:$3/;
+
+    # Store the provided seconds.
+    my ($seconds) = $dt_string =~ /\d\d:\d\d:(\d\d)/;
+    $seconds = '' unless defined $seconds;
+
+    # Insert :00 seconds on time when paired with a TZ offset.
+    $dt_string =~ s/T(\d\d:\d\d)([\-\+Z])/T$1:00$2/;
+    $dt_string =~ s/^(\d\d:\d\d)([\-\+Z])/$1:00$2/;
+
+    # Zero-pad hours when only a single-digit hour appears.
+    $dt_string =~ s/T(\d)$/T0$1/;
+    $dt_string =~ s/T(\d):/T0$1:/;
+
+    # Insert :00 minutes on time when only an hour is listed.
+    $dt_string =~ s/T(\d\d)$/T$1:00/;
+
+    # Treat a space separator between date & time as a 'T'.
+    $dt_string =~ s/ /T/;
+
+    # If this is a time with no date, try to apply a previously-seen
+    # date to it.
+    my $date_is_defined = 1;
+    if ( $dt_string =~ /^\d\d:/ ) {
+        if ( my $previous_dt = $current_item->last_seen_date ) {
+            $dt_string = $previous_dt->ymd . "T$dt_string";
+        }
+        else {
+            $date_is_defined = 0;
+            carp "Encountered a value-class datetime with only a time, "
+                 . "no date, and no date defined earlier. Results may "
+                 . "not be what you expect. (Data: $dt_string)";
+        }
+    }
+
+    eval {
+    $dt = DateTime::Format::ISO8601->new
+              ->parse_datetime( $dt_string );
+    };
+
+    return if $@;
+
+    if ($date_is_defined) {
+        $current_item->last_seen_date( $dt );
+    }
+
+    if ($am_or_pm =~ /^[pP]/) {
+        # There was a 'pm' specified, so add 12 hours.
+        $dt->add( hours => 12 );
+    }
+
+    my $format;
+    if ( ($dt_string =~ /-/) && ($dt_string =~ /[ T]/) ) {
+        my $offset;
+        if ($provided_offset eq 'Z') {
+            $offset = 'Z';
+        }
+        elsif ($provided_offset) {
+            $offset = '%z';
+        }
+        else {
+            $offset = '';
+        }
+        $seconds = ":$seconds" if length $seconds;
+        $format = "%Y-%m-%d %H:%M$seconds$offset";
+    }
+    elsif ( $dt_string =~ /-/ ) {
+        $format = '%Y-%m-%d';
+    }
+
+    return $dt->strftime( $format );
+}
+
+sub _parse_property_value {
+    my ( $self, $element ) = @_;
+
+    my $value;
+
+    my $vcp_fragments_ref =
+        $self->_seek_value_class_pattern( $element );
+    if ( @$vcp_fragments_ref ) {
+        $value = join q{}, @$vcp_fragments_ref;
+    }
+    elsif ( my $alt = $element->findvalue( './@title|@value|@alt' ) ) {
+        $value = $alt;
+    }
+    elsif ( my $text = _trim( decode_entities($element->as_text) ) ) {
+        $value = $text;
+    }
+
+    return $value;
+}
+
+sub _add_element_rels_to_mf2_document {
+    my ( $self, $element, $document ) = @_;
+
+    return unless $element->tag =~ /^(a|link)$/;
+
+    my $rel = $element->attr( 'rel' );
+    return unless defined $rel;
+
+    my $href = $element->attr( 'href' );
+    my $url = URI->new_abs( $href, $self->url_context)->as_string;
+
+    my @rels = split /\s+/, $rel;
+    for my $rel ( @rels ) {
+        $document->add_rel( $rel, $url );
+    }
+
+    my $rel_url_value = {};
+    foreach (qw( hreflang media title type ) ) {
+        next if defined $rel_url_value->{ $_ };
+        my $value = $element->attr( $_ );
+        if ( defined $value ) {
+            $rel_url_value->{ $_ } = $value;
+        }
+    }
+    my $text = ($element->as_text);
+    if ( defined $text ) {
+        $rel_url_value->{ text } = $text;
+    }
+
+    $rel_url_value->{ rels } = \@rels;
+
+    $document->add_rel_url( $url, $rel_url_value );
+
+}
+
 1;
 
 =pod
